
library("gsl")

approxLaplace <- function(x,tau,M) 
{
	log.val1 <- 0.5*log(tau) + lgamma(M+0.5) - log(2) - lgamma(M) - 0.5*log(M)
	val1 <- exp(log.val1)*hyperg_1F1(M+0.5, 0.5, 0.25*tau*x*x/M)
	val2 <- 0.5*sqrt(tau)*abs(x)*hyperg_1F1(M+1, 1.5, 0.25*tau*x*x/M)
	return(val1 - val2)
}

exactLaplace <- function(x,tau)
{
	val <- (0.5/tau)*exp( - tau*abs(x))
}

tau <- 1
x <- seq(-10,10,,600)

q1 <- approxLaplace(x,tau,M=1)
q2 <- approxLaplace(x,tau,M=10)
q3 <- approxLaplace(x,tau,M=30)

p <- exactLaplace(x,tau)

png("ShenDensityFig.png",res=144,width=1200,height=450,antialias="default")	
 
par(mfrow=c(1,3),bty="l")

xlim  <- c(-6,6)
ylim <- c(0,0.7)

plot(NA,type="n",xlim=xlim,ylim=ylim,ylab="p(x;M)",xlab="x",main="M=1",cex.main=1.5,cex.lab=1.5)

lines(x,p,col="lightblue",lwd=6)
lines(x,q1,col="blue",lwd=2)
lines(c(-10,10),c(0,0))

legend("topleft",legend=c("exact Laplace","approximation"),col=c("lightblue","blue"),lwd=c(6,2))

plot(NA,type="n",xlim=xlim,ylim=ylim,ylab="p(x;M)",xlab="x",main="M=10",cex.main=1.5,cex.lab=1.5)

lines(x,p,col="lightblue",lwd=6)
lines(x,q2,col="blue",lwd=2)
lines(c(-10,10),c(0,0))

plot(NA,type="n",xlim=xlim,ylim=ylim,ylab="p(x;M)",xlab="x",main="M=30",cex.main=1.5,cex.lab=1.5)

lines(x,p,col="lightblue",lwd=6)
lines(x,q3,col="blue",lwd=2)
lines(c(-10,10),c(0,0))

dev.off()


