######################## R script: InferNETSupport #############################

# Script containing support functions for the R/InferNET code for the paper:
# 
# Luts J, Shen SSJ, Ormerod JT and Wand MP (2018)
#    Semiparametric Regression Analysis via Infer.NET

################################################################################ 

# Load packages

library("BRugs")
library("SemiPar")
library("KernSmooth")
library("grid")
library("plotrix")
library("matrixcalc")
library("MASS")
library("lattice")
library("sp")
library("mvtnorm")

source("approximateInferenceFunctions.R")
source("mixFunctions.R")
source("splineFunctions.R")

################################################################################ 

# RunInferNET - function to simplify calls to InferNET

# InferNETSupportDirectory - Location of directory where support files are
# located InferNETCSfile - Name of the InferNET C# (C sharp) file
# MSBuildDirectory - Location of directory where MSBuild resides Verbose - Output
# compilation information removeGeneratedSource - Remove files associated with
# the compilation

RunInferNET <- function(InferNETSupportDirectory, InferNETCSfile, MSBuildDirectory = "C:\\Windows\\Microsoft.NET\\Framework64\\v4.0.30319\\", 
  verbose = FALSE, removeGeneratedSource = TRUE) {
  # Copy C# code to the InferNETSupport directory:
  file.copy(InferNETCSfile, paste(InferNETSupportDirectory, "InferNETSupport\\RunMe.cs", 
    sep = ""), overwrite = TRUE)
  
  fileName1 <- paste(MSBuildDirectory, "MsBuild.exe", sep = "")
  projectName1 <- paste(InferNETSupportDirectory, "InferNETSupport.sln", sep = "")
  
  if (verbose) {
    paramName1 <- "/t:rebuild /p:configuration=Debug"
  } else {
    paramName1 <- "/t:rebuild /p:configuration=Debug /v:q"
  }
  system(paste(fileName1, projectName1, paramName1))
  system(paste(InferNETSupportDirectory, "InferNETSupport\\bin\\Debug\\InferNETSupport.exe", 
    sep = ""))
  
  if (removeGeneratedSource) {
    unlink(paste(getwd(), "\\GeneratedSource", sep = ""), recursive = TRUE)
  }
}

######################## R script: InferNETSupport #############################
 
