///////////////////// C# class: DataTable /////////////////

// For storing a data table for use in a C# programme
// involving use of Infer.NET.

// Last changed: 04 MAR 2011 by S.S.J. Wang and M.P. Wand

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.IO;
using MicrosoftResearch.Infer.Maths;

namespace LutsWangOrmerodWand
{
   class DataTable
   {

      // Claim a public double precision two-dimensional array 
      // named "DataMatrix":

      public readonly double[,] DataMatrix;

      // Claim a public double predicision one-dimensional array 
      // named "arrayForIN" for use in Infer.Net:

      public readonly double[] arrayForIN;

      // Claim a public vector array named "arrayVectorForIN" 
      // for use in Infer.Net:
     
      public readonly Vector[] arrayVectorForIN;

      private string[] header;
      
      // Claim an integer named "numCol":
      
      public readonly int numCol;
      
      // Claim an integer named "numRow":

      public readonly int numRow;
      
      // Claim a double precision real number named "DoubleNumeric":
      
      public readonly double DoubleNumeric;
      
      // Claim an integer named "IntNumeric":
      
      public readonly int IntNumeric;

      // Set up a building function for "DataTable" that
      // allows for different inputs:

      private DataTable()
      {
         throw new System.NotImplementedException(
            "You need input the address of input file");
      }

      public DataTable(string path, char sep, bool haveheader)
      {
         char[] separate = { ' ', ',', sep };
         LoadData(
            path, separate, haveheader, out DataMatrix, 
            out arrayForIN, out arrayVectorForIN, 
            out DoubleNumeric, out header,out IntNumeric, 
            out numCol, out numRow);

      }

      public DataTable(string path, char sep)
      {
         char[] separate = { ' ', ',', sep };
         bool haveheader = false;
         LoadData(
            path, separate, haveheader, out DataMatrix, 
            out arrayForIN, out arrayVectorForIN, 
            out DoubleNumeric, out header, out IntNumeric, 
            out numCol, out numRow);
      }

      public DataTable(string path, bool haveheader)
      {
         char[] separate = { ' ', ',' };
         LoadData(
            path, separate, haveheader, out DataMatrix, 
            out arrayForIN, out arrayVectorForIN, 
            out DoubleNumeric, out header, out IntNumeric, 
            out numCol, out numRow);
      }

      public DataTable(string path)
      {
         char[] separate = { ' ', ',' };
         bool haveheader = false;
         LoadData(
            path, separate, haveheader, out DataMatrix, 
            out arrayForIN, out arrayVectorForIN, 
            out DoubleNumeric, out header, out IntNumeric, 
            out numCol, out numRow);
      }

      // Claim a string named "Header":

      public string[] Header
      {
         get
         {
            return header;
         }
         set
         {
            header = value;
         }
      }
      
      // Read in data from the specified file:

      private void LoadData(
         string path, char[] sep, bool haveheader, 
         out double[,] data, out double[] arrayData, 
         out Vector[] arrayVector, out double doublenumeric, 
         out string[] header, out int intnumeric, 
         out int ncol, out int nrow)
      {
         StreamReader objReader = new StreamReader(path);
         string sLine = "";
         ArrayList arrText = new ArrayList();

         while (sLine != null)
         {
            sLine = objReader.ReadLine();
            if (sLine != null)
               arrText.Add(sLine);
         }
         objReader.Close();

         nrow = arrText.Count;

         string[] firstline;
         firstline = arrText[0].ToString().Replace("\"", "").
            Split(sep, System.StringSplitOptions.RemoveEmptyEntries);
         ncol = firstline.Length;

         if (haveheader)
            header = firstline;
         else
            header = new string[ncol];

         string[] SplitStr;
         int lines = 0;
         if (haveheader)
         {
            lines--;
            nrow--;
         }

         data = new double[nrow, ncol];
         foreach (string outputStr in arrText)
         {
            if (lines > -1)
            {
               SplitStr = outputStr.Split(
                  sep,System.StringSplitOptions.RemoveEmptyEntries);
               try
               {
                  for (int v = 0; v < ncol; v++)
                     data[lines, v] = double.Parse(SplitStr[v]);
               }
               catch
               {
                  throw new System.NotImplementedException(
        "The file probably contains non-numeric information on the first line.");
               }

            }
            lines++;
         }

         arrayData = new double[nrow];
         for (int i = 0; i < nrow; i++)
         {
            arrayData[i] = data[i, 0];
         }

         arrayVector = new Vector[nrow];
         for (int i = 0; i < nrow; i++)
         {
            arrayVector[i] = Vector.Zero(ncol);
            for (int j = 0; j < ncol; j++)
            {
               arrayVector[i][j] = data[i, j];
            }
         }

         doublenumeric = data[0,0];
         intnumeric = Convert.ToInt32(data[0,0]);
      }
   }
}

///////////////////// End of DataTable /////////////////
