///////////////////// C# class: SaveData /////////////////

// For saving Infer.NET output data.

// Last changed: 04 MAR 2011 by S.S.J. Wang and M.P. Wand.

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.IO;
using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Models;

namespace LutsWangOrmerodWand
{
   class SaveData
   {
      private static void saveData(string args, string filename)
      {
         try
         {
            FileStream aFile = new FileStream(filename, FileMode.Create);
            StreamWriter sw = new StreamWriter(aFile);
            sw.WriteLine(args);
            sw.Close();
            aFile.Close();
            Console.WriteLine("The output file is " + filename);

         }
         catch (IOException e)
         {
            Console.WriteLine(e.ToString());
         }
      }

      private static void SaveArrayGamma(string arraygamma, string filename)
      {
         arraygamma = arraygamma.Replace("[", "");
         arraygamma = arraygamma.Replace("]", ",");
         arraygamma = arraygamma.Replace("Gamma(", "");
         arraygamma = arraygamma.Replace(")mean=", ",");
         saveData(arraygamma, filename);
      }

      private static void SaveGamma(string gamma, string filename)
      {
         gamma = gamma.Replace("Gamma(", "");
         gamma = gamma.Replace(")[mean=", ", ");
         gamma = gamma.Replace("]", "");
         gamma = gamma.Replace(",", "");
         saveData(gamma, filename);
      }

      private static void SaveNumeric(string numeric, string filename)
      {
         saveData(numeric, filename);
      }

      private static void SaveArrayGaussian(string arraygaussian, string filename)
      {
         arraygaussian = arraygaussian.Replace("[", "");
         arraygaussian = arraygaussian.Replace("]", ",");
         arraygaussian = arraygaussian.Replace("Gaussian(", "");
         arraygaussian = arraygaussian.Replace(")", "");
         saveData(arraygaussian, filename);
      }

      private static void SaveDirichlet(string dirichlet, string filename)
      {
         dirichlet = dirichlet.Replace("Dirichlet(", "");
         dirichlet = dirichlet.Replace(")", "");
         saveData(dirichlet, filename);
      }

      private static void SaveArrayVectorGaussian(string arrayvectorgaussian, string filename)
      {
         
         int n = 0;
         do
         {
            arrayvectorgaussian = arrayvectorgaussian.Replace("[" + n.ToString() + "]", "");
            n++;
         } while (arrayvectorgaussian.Contains("[" + n.ToString() + "]"));
         arrayvectorgaussian = arrayvectorgaussian.Replace("VectorGaussian(", "");
         do
         {
            arrayvectorgaussian = arrayvectorgaussian.Replace("  ", " ");
         } while (arrayvectorgaussian.Contains("  "));
         arrayvectorgaussian = arrayvectorgaussian.Replace(",", "\r\n");
         arrayvectorgaussian = arrayvectorgaussian.Replace(")", "");
         saveData(arrayvectorgaussian, filename);
      }

      private static void SaveArrayWishart(string arraywishart, string filename)
      {

         int n = 0;
         do
         {
            arraywishart = arraywishart.Replace("[" + n.ToString() + "]", "");
            n++;
         } while (arraywishart.Contains("[" + n.ToString() + "]"));
         arraywishart = arraywishart.Replace("Wishart(", "");
         do
         {
            arraywishart = arraywishart.Replace("  ", " ");
         } while (arraywishart.Contains("  "));
         arraywishart = arraywishart.Replace(",", "\r\n");
         arraywishart = arraywishart.Replace(")", "");
         saveData(arraywishart, filename);
      }

      private static void SaveWishart(string wishart, string filename)
      {
         wishart = wishart.Replace("Wishart(", " ");
         do
         {
            wishart = wishart.Replace("  ", " ");
         } while (wishart.Contains("  "));
         wishart = wishart.Replace(",", "\r\n");
         wishart = wishart.Replace(")", "");
         saveData(wishart, filename);
      }

      public static void SaveTable(Gamma data, string filename)
      {
         SaveGamma(data.ToString(),filename);
      }

      public static void SaveTable(Dirichlet data, string filename)
      {
         SaveDirichlet(data.ToString(),filename);
      }

      public static void SaveTable(
         DistributionStructArray<Gaussian, double> data, 
         string filename)
      {
         SaveArrayGaussian(data.ToString(),filename);
      }

      public static void SaveTable(
         DistributionRefArray<Wishart, PositiveDefiniteMatrix> data, 
         string filename)
      {
         SaveArrayWishart(data.ToString(),filename);
      }

      public static void SaveTable(Wishart data, string filename)
      {
         SaveWishart(data.ToString(),filename);
      }

      public static void SaveTable(
         DistributionRefArray<VectorGaussian,Vector> data, 
         string filename)
      {
         SaveArrayVectorGaussian(data.ToString(), filename);
      }
      public static void SaveTable(
         DistributionStructArray<Gamma, double> data, 
         string filename)
      {
         SaveArrayGamma(data.ToString(),filename);
      }

      public static void SaveTable(Vector data, string filename)
      {
         SaveNumeric(data.ToString(),filename);
      }

      public static void SaveTable(PositiveDefiniteMatrix data, string filename)
      {
         SaveNumeric(data.ToString(),filename);
      }

      public static void SaveTable(int data, string filename)
      {
         SaveNumeric(data.ToString(),filename);
      }

      public static void SaveTable(double data, string filename)
      {
         SaveNumeric(data.ToString(),filename);
      }
   }
}

///////////////////// End of SaveData /////////////////
