########## R-function: dnorMix ##########

# For computing the ordinate values of a normal mixture density
# function.

dnorMix <- function(x, wVec, meanVec, sdVec) {
  k <- length(wVec)
  ans <- 0
  for (j in 1:k) {
    newTerm <- wVec[j] * dnorm(x, meanVec[j], sdVec[j])
    ans <- ans + newTerm
  }
  return(ans)
}

########## End of dnorMix ############

########## R-function: pnorMix ##########

# For computing the ordinate values of a # normal mixture distribution
# function.

pnorMix <- function(x, wVec, meanVec, sdVec, drv = 0) {
  k <- length(wVec)
  ans <- 0
  for (j in 1:k) {
    newTerm <- wVec[j] * pnorm(x, meanVec[j], sdVec[j])
    ans <- ans + newTerm
  }
  return(ans)
}

########## End of pnorMix ############

########## R-function: qnorMix ##########

# Computes values the quantile function of a normal mixture distribution.

qnorMix <- function(p, wVec, meanVec, sdVec) {
  if ((p < 0) | (p > 1)) 
    stop("p must be between 0 and 1.")
  if ((sum(wVec) > 1.000001) | (sum(wVec) < 0.999999)) 
    stop("weights must sum to 1.")
  if (any(sdVec < 0)) 
    stop("entries of sdVec must be positive")
  
  # Do bisection search for Newton-Raphson starting value:
  
  qLow <- min(meanVec)
  qUpp <- max(meanVec)
  incremVal <- 1 + abs(qLow - qUpp)/2
  rootCaptured <- FALSE
  while (!rootCaptured) {
    fLow <- pnorMix(qLow, wVec, meanVec, sdVec) - p
    fUpp <- pnorMix(qUpp, wVec, meanVec, sdVec) - p
    if (fLow > 0) 
      qLow <- qLow - incremVal
    if (fUpp < 0) 
      qUpp <- qUpp + incremVal
    if ((fLow < 0) & (fUpp > 0)) 
      rootCaptured <- TRUE
  }
  numBisect <- 100
  for (ib in 1:numBisect) {
    qMid <- (qLow + qUpp)/2
    fLow <- pnorMix(qLow, wVec, meanVec, sdVec) - p
    fUpp <- pnorMix(qUpp, wVec, meanVec, sdVec) - p
    fMid <- pnorMix(qMid, wVec, meanVec, sdVec) - p
    if (fMid < 0) 
      qLow <- qMid
    if (fMid > 0) 
      qUpp <- qMid
  }
  qOld <- qMid
  
  # Do Newton-Raphson iterations:
  
  maxIter <- 1000
  tolVal <- 1e-06
  converged <- FALSE
  itNum <- 0
  while (!converged) {
    itNum <- itNum + 1
    numer <- pnorMix(qOld, wVec, meanVec, sdVec) - p
    denom <- dnorMix(qOld, wVec, meanVec, sdVec)
    qNew <- qOld - numer/denom
    relErr <- abs((pnorMix(qNew, wVec, meanVec, sdVec)/pnorMix(qOld, wVec, meanVec, 
      sdVec)) - 1)
    if (relErr < tolVal) 
      converged <- TRUE
    if (itNum > maxIter) 
      stop("Failed to converge after ", maxIter, " iterations.")
    qOld <- qNew
  }
  return(qNew)
}

########## End of qnorMix ############

########## R function: logOfSum ##########

# Computes the log of a sum

logOfSum <- function(array) {
  term <- 1
  
  for (i in 2:length(array)) {
    term <- term + exp(array[i] - array[1])
  }
  
  term <- array[1] + log(term)
  
  return(term)
}

############ End of logOfSum ###########
 
